local mod = MorgDKP2:NewModule("Zerosum")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local media = LibStub("LibSharedMedia-3.0", true)
local BC = LibStub("LibBabble-Class-3.0"):GetLookupTable()

local mdkp = MorgDKP2

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("Zerosum", "enUS", true)
	
if LL then
	LL["Zerosum DKP"] = true
	LL["Enable"] = true
	LL["Standard zero sum system where member is charged for the item and the charged points are divided between all raid members."] = true
	LL["Include waitlist"] = true
	LL["Include waitlist members in DKP calculations.  Note they will still get whatever DKP award is set in the waitlist module."] = true
	LL["Added %s DKP to %s attendees."] = true
	LL["DKP award event name"] = true
	LL["Format of event name:  \n<zone> text  \nExample:  <zone> DKP"] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("Zerosum")

mod.modName = LL["Zerosum DKP"]
mod.modref = "Zerosum"

local Player = UnitName("player")
local db
local fmt = string.format
local floor = math.floor
local tinsert = table.insert

local defaults = { 
	profile = {
		usewait = nil,
	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Zerosum DKP"],
		desc = LL["Zerosum DKP"],
		order = 360,
		args = {
			enable = {
				type = "toggle",
				name = LL["Enable"],
				desc = LL["Standard zero sum system where member is charged for the item and the charged points are divided between all raid members."],
				get = function() return mdkp.db.profile.modules.Zerosum end,
				set = 	function(info, v) 
						mdkp.db.profile.modules.Zerosum = v
						if v then 
							--mod:Enable()
							mod:DKPChange(mod.modName)
						else 
							mod:Disable()
							mdkp.db.profile.currDKP = nil 	
						end
					end,
				order = 5
			},
			waitlist = {
				type = "toggle",
				name = LL["Include waitlist"],
				desc = LL["Include waitlist members in DKP calculations.  Note they will still get whatever DKP award is set in the waitlist module."],
				get = function() return mod.db.profile.usewait end,
				set = function(info, v) mod.db.profile.usewait = v end,
				hidden = function() return not mdkp.db.profile.modules.Zerosum end,
				order = 10
			},
		},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return true
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("Zerosum", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["Zerosum"] = true
	
end

function mod:OnDisable()
	db.moduleON["Zerosum"] = nil
	--if db.currDKP == "Zerosum" then db.currDKP = nil end						
end

function mod:GetPlayerPoints(name, POOL)
	return db.info[name][POOL].points
end

function mod:GetItemValue(itemid, player, POOL)
	local itempoints = db.items[itemid].points
	if db.takemode or db.offspecmode then
		local take = db.takeamount
		local offspec = db.offspecamount
		if mdkp.querytooltips[itemid] then
			for  i,v in pairs(mdkp.querytooltips[itemid]) do
				if v[1].n == player then
					if db.takemode and (v[2].n == db.TAKE or v[2].n == L["TAKE"]) then 
						if db.takepercent then itempoints = itempoints * take / 100
						else itempoints = take end
					elseif db.offspecmode and (v[2].n == db.OFFSPEC or v[2].n == L["OFFSPEC"]) then 
						if db.takepercent then itempoints = itempoints * offspec / 100
						else itempoints = offspec end
					end
					break
				end
			end
		end
	end
	return itempoints
end

function mod:ItemRecorded(id, looter, POOL, points, itempoints, link, attendees, boss)
	if looter == L["Bank"] or not attendees then return end
	points = points - itempoints
	self:AwardItem(id, looter, POOL, points, itempoints, link)
	
	local zeroattends = {}
	for _, name in pairs(attendees) do
		tinsert(zeroattends, name)
	end
	
	--add zero sum points
	if itempoints == 0 or not attendees then return end
	if self.db.profile.usewait then
		local waitattends
		local killid = self:GetEventID(2)
		if boss ~= "Trash mob" then killid = #db.raidlog[db.raidnum].bosskills end
		local wait = self:GetModuleRef("Waitlist")
		if wait then waitattends = wait:GetWaitlist() end
		if waitattends then
			for _, name in pairs(waitattends) do
				tinsert(zeroattends, name)
				local found = nil
				for _, inname in pairs(db.raidlog[db.raidnum].bosskills[killid].attendees) do
					if inname == name then
						found = true
						break
					end
				end
				if not found then tinsert(db.raidlog[db.raidnum].bosskills[killid].attendees, name) end
			end
		end
	end
	addpoints = floor(itempoints / #zeroattends * 100) / 100
	self:AwardDKP(addpoints, 0, POOL, zeroattends)
end

function mod:ItemRebate(id, looter, POOL, itempoints, link, oldattends, newattends)
	if newattends then		--changing item event
		--remove points from old attendees
		if oldattends then
			local addpoints = itempoints/#oldattends
			self:AwardDKP(0, addpoints, POOL, oldattends)
		end
		
		--add points to new attendees
		local addpoints = itempoints/#newattends
		self:AwardDKP(addpoints, 0, POOL, newattends)
	else
		if oldattends and looter ~= L["Bank"] then
			local addpoints = itempoints/#oldattends
			self:AwardDKP(0, addpoints, POOL, oldattends)
			self:AdjustDKPPoints(looter, looter, POOL, nil, itempoints)
		end
	end
end

function mod:ItemAdjustment(id, looter, POOL, newcost, link, oldcost, attends)
	local diff = oldcost - newcost
	local addpoints = diff/#attends
	self:AwardDKP(0, addpoints, POOL, attends)
end


function mod:ItemTransfer(id, newmember, oldmember, POOL, newcost, link, oldcost)
	self:AdjustDKPPoints(newmember, oldmember, POOL, newcost, oldcost, link)
end
